@@ -34,7 +34,7 @@
 //! println!("Second cell has site {:?}, voronoi vertices {:?} and delaunay triangles {:?}",
 //!     my_cell.site_position(),
 //!     my_cell.iter_vertices().collect::<Vec<&Point>>(),
-//!     my_cell.iter_triangles().collect::<Vec<usize>>());
+//!     my_cell.triangles().iter().collect::<Vec<&usize>>());
 //!
 //! // or, for graphical applications, that benefit from index buffers
 //! // you can access the raw, indexed data
@@ -51,15 +51,18 @@ mod iterator;
 mod utils;
 mod voronoi_builder;
 
+use std::{fmt::Display, str::FromStr};
+
 use delaunator::{EMPTY, Triangulation, triangulate};
 use self::{
-    utils::{cicumcenter, site_of_incoming},
+    utils::cicumcenter,
     cell_builder::*
 };
 
 pub use voronoi_builder::VoronoiBuilder;
 pub use bounding_box::BoundingBox;
 pub use voronoi_cell::VoronoiCell;
+pub use iterator::TopologicalNeighborSiteIterator;
 pub use iterator::NeighborSiteIterator;
 pub use iterator::CellPathIterator;
 pub use delaunator::Point;
@@ -85,9 +88,29 @@ impl Default for ClipBehavior {
     }
 }
 
+impl Display for ClipBehavior {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{:?}", self)
+    }
+}
+
+impl FromStr for ClipBehavior {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "None" => Ok(Self::None),
+            "RemoveSitesOutsideBoundingBoxOnly" => Ok(Self::RemoveSitesOutsideBoundingBoxOnly),
+            "Clip" => Ok(Self::Clip),
+            _ => Err("Invalid option".to_string())
+        }
+    }
+}
+
 /// The dual Delaunay-Voronoi graph.
 ///
 /// To obtain an instance of this type, use [VoronoiBuilder].
+#[derive(Clone)]
 pub struct Voronoi {
     /// These are the sites of each voronoi cell.
     sites: Vec<Point>,
@@ -138,8 +161,6 @@ impl Voronoi {
         // triangulation.triangles is the indexing of each half-edge to source site
         // 3 * t, 3 * t + 1 and 3 * t + 2 are the vertices of a triangle in this vector
         let num_of_triangles = triangulation.triangles.len() / 3;
-        let num_of_sites = sites.len();
-
         if num_of_triangles == 0 {
             return None
         }
@@ -151,28 +172,13 @@ impl Voronoi {
             &sites[triangulation.triangles[3* t + 2]])
         ).collect();
 
-        // create map between site and its left-most incoming half-edge
-        // this is especially important for the sites along the convex hull boundary when iterating over its neighoring sites
-        let mut site_to_incoming_leftmost_halfedge = vec![EMPTY; num_of_sites];
-
-        for e in 0..triangulation.triangles.len() {
-            let s = site_of_incoming(&triangulation, e);
-            if site_to_incoming_leftmost_halfedge[s] == EMPTY || triangulation.halfedges[e] == EMPTY {
-                site_to_incoming_leftmost_halfedge[s] = e;
-            }
-        }
-
-        // if input sites has lot of coincident points (very very close), the underlying triangulation will be problematic and those points may be ignored in the triangulation
-        // thus they won't be reacheable and this will lead to problems down the line as we build the voronoi graph
-        debug_assert!(!site_to_incoming_leftmost_halfedge.iter().any(|e| *e == EMPTY), "One or more sites is not reacheable in the triangulation mesh. This usually indicate coincident points.");
-
         // create cell builder to build cells and update circumcenters
-        let cell_builder = CellBuilder::new(circumcenters, bounding_box.clone(), clip_behavior);
-        let result = cell_builder.build(&sites, &triangulation, &site_to_incoming_leftmost_halfedge);
+        let cell_builder = CellBuilder::new(&triangulation, &sites, circumcenters, bounding_box.clone(), clip_behavior);
+        let result = cell_builder.build();
 
         Some(Voronoi {
             bounding_box,
-            site_to_incoming_leftmost_halfedge,
+            site_to_incoming_leftmost_halfedge: result.site_to_incoming_leftmost_halfedge,
             triangulation,
             sites,
             clip_behavior,
@@ -247,11 +253,13 @@ impl Voronoi {
         &self.circumcenters
     }
 
-    /// Gets a reference to a vector of indices to sites where each triple represents a triangle on the dual Delaunay triangulation associated with this Voronoi graph.
+    /// Gets a reference to the underlying delaunay triangulation.
+    ///
+    /// ```triangulation().triangles``` is a vector of indices to sites where each triple represents a triangle on the dual Delaunay triangulation associated with this Voronoi graph.
     /// All triangles are directed counter-clockwise.
     #[inline]
-    pub fn delaunay_triangles(&self) -> &Vec<usize> {
-        &self.triangulation.triangles
+    pub fn triangulation(&self) -> &Triangulation {
+        &self.triangulation
     }
 
     /// Gets a reference to the bounding box.
@@ -275,8 +283,8 @@ mod tests {
         let builder = VoronoiBuilder::default();
         let bbox = BoundingBox::default();
 
-        let x_range = rand::distributions::Uniform::new(-bbox.width(), bbox.width());
-        let y_range = rand::distributions::Uniform::new(-bbox.height(), bbox.height());
+        let x_range = rand::distributions::Uniform::new(-bbox.width() / 2.0, bbox.width() / 2.0);
+        let y_range = rand::distributions::Uniform::new(-bbox.height() / 2.0, bbox.height() / 2.0);
         let sites = (0..size)
             .map(|_| Point { x: rng.sample(x_range), y: rng.sample(y_range) })
             .collect();
@@ -287,10 +295,47 @@ mod tests {
     }
 
     #[test]
-    fn random_site_generation_test() {
-        create_random_builder(100_000)
+    fn random_100_000_site_generation_test() {
+        let voronoi = create_random_builder(100_000)
             .build()
             .expect("Some voronoi expected.");
+
+        utils::test::validate_voronoi(&voronoi);
+    }
+
+    #[test]
+    fn random_15_site_generation_test() {
+        for _ in 0..1000 {
+            let voronoi = create_random_builder(15)
+                .build()
+                .expect("Some voronoi expected.");
+
+            utils::test::validate_voronoi(&voronoi);
+        }
+    }
+
+    #[test]
+    fn validate_examples() -> std::io::Result<()> {
+        for path in [
+            "degenerated1.json",
+            "degenerated2.json",
+            "degenerated3.json",
+            "degenerated4.json",
+            "degenerated5.json",
+            "degenerated6.json",
+            "degenerated7.json",
+            "degenerated8.json",
+            "degenerated9.json",
+            "degenerated10.json",
+            "clockwise1.json",
+        ] {
+            let voronoi = utils::test::new_voronoi_builder_from_asset(path)?
+                .build()
+                .expect("Some voronoi expected");
+            utils::test::validate_voronoi(&voronoi);
+        }
+
+        Ok(())
     }
 
     #[test]
